@@ -2,7 +2,7 @@
 from cbor2 import dump
 from dataclasses import dataclass, asdict
 from typing import List, Optional, Mapping
-import numpy
+import numpy as np
 import torch
 import struct
 from time import time
@@ -38,6 +38,16 @@ class DEEPHYSData:
 
 @dataclass
 class Model(DEEPHYSData):
+    """
+
+    :param name: name of the model (e.g.: `"resnet18"`)
+    :type name: str
+    :param suffix: optional suffix for the model
+    :type suffix: Optional[str]
+    :param layers: A List of layers in the model. The app is guaranteed to support cases where there are 2 layers and one of them is called `"classification"`.
+    :type layers: List[deephys.deephys.Layer]
+    """
+
     layers: List[Layer]
 
     def __post_init__(self):
@@ -73,8 +83,8 @@ def import_torch_dataset(name, dataset, classes, state, model):
     :type dataset: torch.utils.data.DataLoader
     :param classes: an ordered list of strings representing class names
     :type classes: list
-    :param state: a 3D array of floats [layers,neurons,activations]
-    :type state: list
+    :param state: a 3D array of floats [layers,neurons,activations]. Length of activations must be the same as the number of images.
+    :type state: Union[list,np.ndarray]
     :param model: the model structure
     :type model: deephys.deephys.Model
     :return: a formatted data object which may be saved to a file
@@ -88,6 +98,7 @@ def import_torch_dataset(name, dataset, classes, state, model):
             target = target.item()
         pixelDataList.append(image)
         groundTruthList.append(target)
+    pixelDataList = torch.stack(pixelDataList)
     return import_test_data(
         name=name,
         pixel_data=pixelDataList,
@@ -98,6 +109,12 @@ def import_torch_dataset(name, dataset, classes, state, model):
     )
 
 
+def _to_mat(d):
+    if isinstance(d, list):
+        return np.array(d)
+    return d
+
+
 def import_test_data(name, classes, state, model, pixel_data, ground_truths):
     """
     Prepare test results for Deephys
@@ -106,36 +123,41 @@ def import_test_data(name, classes, state, model, pixel_data, ground_truths):
     :type name: str
     :param classes: an ordered list of strings representing class names
     :type classes: list
-    :param state: a 3D array of floats [layers,neurons,activations]
-    :type state: list
+    :param state: a 3D array of floats [layers,neurons,activations]. Length of activations must be the same as the number of images.
+    :type state: Union[list,np.ndarray]
     :param model: the model structure
     :type model: deephys.deephys.Model
-    :param pixel_data: an ordered list of image pixel data [images,channels,dim1,dim2]
-    :type pixel_data: List[List[List[List]]]
+    :param pixel_data: an ordered list of image pixel data [images,channels,dim1,dim2]. Pixels must be floats within the range 0.0:1.0
+    :type pixel_data: Union[list,np.ndarray,torch.FloatTensor]
     :param ground_truths: an ordered list of ground truths
     :type ground_truths: List[int]
     :return: a formatted data object which may be saved to a file
     :rtype: deephys.deephys.Test
     """
     imageList = []
+    state = list(map(_to_mat, state))
+    if isinstance(pixel_data, list):
+        pixel_data = torch.tensor(pixel_data)
+    elif isinstance(pixel_data, np.ndarray):
+        pixel_data = torch.from_numpy(pixel_data)
+    if len(ground_truths) != len(pixel_data):
+        raise Exception(
+            f"ground_truths length ({len(ground_truths)}) must be same as the image length ({len(pixel_data)})"
+        )
     for i in range(len(pixel_data)):
         image = pixel_data[i]
         target = ground_truths[i]
-        image = image * 255
         mn = torch.min(image)
         mx = torch.max(image)
-        if mx > 255:
+        if mx > 1 or mn < 0:
             raise Exception(
-                f"image pixel values should be integers between 0 and 255, but a value of {mx} was received"
-            )
-        if mn < 0:
-            raise Exception(
-                f"image pixel values should be integers between 0 and 255, but a value of {mn} was received"
+                f"image pixel values must be floats between 0 and 1, but values given range from {mn} to {mx}"
             )
+        image = image * 255
         chan_to_bytes = lambda chan: [bytes(row) for row in chan]
         im_to_bytes = lambda im: list(map(chan_to_bytes, im))
-        im_as_list = image.numpy().astype(numpy.uint8).tolist()
-        im_activations = list(map(lambda x: x[i, :].tolist(), state))
+        im_as_list = image.numpy().astype(np.uint8).tolist()
+        im_activations = list(map(lambda l: l[:, i], state))
         imageList.append(
             ImageFile(
                 imageID=i,
